-- cl560 simple model to host interface
library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

entity cl560 is
    port (
-- the following signals are for generating psuedo dma requests from cl560
	adr : inout STD_LOGIC_VECTOR (31 downto 0);
	data : inout STD_LOGIC_VECTOR (31 downto 0); 
	dma : inout std_logic; -- 1 run dma mode
	dma_cnt : inout std_logic_vector(7 downto 0); -- dma count+1
	not_r_w : in std_logic;	 -- 1=read, 0=write
-- set up the following to insert wait states
	wait_cnt : inout std_logic_vector(7 downto 0);
-- dma_wait_cnt not used
	dma_wait_cnt : inout std_logic_vector(7 downto 0);
-- pixel interface - these do nothing
	not_frmend : out std_logic := '1';
	not_pxin,not_pxout,not_blank : out std_logic := '1';
-- compression interface
	hbclk : in std_logic;
	hbus : inout std_logic_vector(31 downto 0):="ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
	not_start : in std_logic;
	not_tmout : inout std_logic;
	not_hbout : out std_logic;
	not_drq : out std_logic :='1';
	not_irq1 : out std_logic :='1';
	irq2 : out std_logic :='0';
	not_tm : inout std_logic_vector(2 downto 0)
	
   );
end cl560;

library ieee;
use ieee.std_logic_1164.all;
--library exemplar;
--use exemplar.exemplar_1164.all;


architecture operate_cl560 of cl560 is

type bus_state is
	(nop,start,wait_cycle,ack);

signal cl560_state : bus_state := nop;
signal tmack : std_logic :='0';
signal errors : std_logic :='0';
signal r_w_state : std_logic;
signal xnot_tmout: std_logic := '1';

signal xdma_cnt,xwait_cnt : std_logic_vector(7 downto 0);
signal  ladr : Std_logic_vector(31 downto 0);

constant t36 : time := 20 ns;
constant t42 : time := 20 ns;
constant t20 : time := 22 ns;
constant t6 : time := 18 ns;
constant t7 : time := 18 ns;
constant t39 : time := 23 ns;
--constant t38 : time := 2 ns;
constant t40 : time := 23 ns;
constant t41 : time := 5 ns;

constant t26 : time := 8 ns;
constant t27 : time := 12 ns;
constant t16 : time := 8 ns;
constant t17 : time := 12 ns;



begin

assert1: for i in 0 to 31 generate
x : ASSERT not((hbclk='0' and not hbclk'stable) and (not hbus(i)'stable(t26)) and ((cl560_state=start) or ((cl560_state=ack) and (r_w_state = '0'))) ) 
        REPORT "setup violation hbclk to hbus"
        SEVERITY warning;

y : ASSERT not((hbclk='0' and hbus(i)'event) and (not hbclk'stable(t27)) and  ((cl560_state=start) or ((cl560_state=ack) and (r_w_state = '0'))) ) 
        REPORT "hold violation hbclk to hbus"
        SEVERITY warning;

end generate;

x2 : ASSERT not((hbclk='0' and not hbclk'stable) and (not not_start'stable(t16))) 
        REPORT "setup violation hbclk to not_start"
        SEVERITY warning;

y2 : ASSERT not((hbclk='0' and not_start'event) and (not hbclk'stable(t17))) 
        REPORT "hold violation hbclk to not_start"
        SEVERITY warning;


proc_tmout: process (xnot_tmout,tmack,errors)
  begin
    if (xnot_tmout='0') then
	   not_tm(0) <= TRANSPORT ('1' and not tmack) or (tmack and errors) after t20;
	   not_tm(1) <= TRANSPORT ('1' and not tmack) or (tmack and errors) after t20;
	   not_tm(2) <= TRANSPORT not tmack after t20;
      else
	    not_tm(0) <= TRANSPORT 'Z' after t20;
	    not_tm(1) <= TRANSPORT 'Z' after t20; 
	    not_tm(2) <= TRANSPORT 'Z' after t20;
	end if;
  end process proc_tmout;


proc_data: process (tmack,r_w_state,data)
begin
    if ((r_w_state = '1') and (tmack='1')) then -- read
	  hbus <= TRANSPORT data after t39;
    end if;
    if ((r_w_state = '1') and (falling_edge(tmack))) then
      hbus <= TRANSPORT "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" after t41;
      hbus <= TRANSPORT "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ" after t40;
    end if;
 end process proc_data;


proc_op:			-- optional label
process --(dma,not_start,hbclk,not_tm(1),not_tm(2),r_w_state,xwait_cnt,data)
-- declarations
begin
--  wait on dma,not_start,hbclk,not_tm(1),not_tm(2),reg_r_w_state,xwait_cnt,data;
  case cl560_state is
  when nop =>
	xwait_cnt <= wait_cnt;
    not_hbout <= TRANSPORT '1' after t42;
    not_tmout <= TRANSPORT '1' after t36;
    xnot_tmout <= '1';
    tmack <= '0'; 
    wait until (falling_edge(not_start)) ;
    wait until (falling_edge(hbclk)) ;
    if (not_tm(2)='1') then
      cl560_state <= start;
	  r_w_state <= not_tm(1);
      ladr <= hbus;
	end if;
  when  start =>
	wait until (rising_edge(hbclk));
    if (r_w_state='1') then -- read
	  not_hbout <= TRANSPORT '0' after t42;
    else -- write
    end if;
	if ((xwait_cnt/="00000000") and (ladr(15)='1')) then -- no wait states on fifo access
	  cl560_state <= wait_cycle;
	else
	  cl560_state <= ack;
	end if;
  when wait_cycle =>
	not_tmout <= TRANSPORT '0' after t36;
	xnot_tmout <= '0';
    xwait_cnt <= xwait_cnt +"11111111";
	wait until (rising_edge(hbclk));
    if (xwait_cnt="00000000") then
      cl560_state <= ack;
    end if;
  when ack =>  
	not_tmout <= TRANSPORT '0' after t36;
	xnot_tmout <= '0';
    tmack <= '1'; -- acknowledge
	wait until (rising_edge(hbclk));
	cl560_state <= nop;
    not_tmout <= TRANSPORT '1' after t36;
    xnot_tmout <= '1';
    not_hbout <= TRANSPORT '1' after t42;
    tmack <= '0'; 
  end case;
  wait for 1 ns;
end process proc_op;

proc_dma: process 
begin
    wait until rising_edge(dma);
    not_irq1 <= '0';
    irq2 <= '1';
    xdma_cnt <= dma_cnt;
    for i in 0 to evec2int(dma_cnt) loop
      wait until rising_edge(xnot_tmout) and ladr(15)='0';
      xdma_cnt <= xdma_cnt +"11111111";
    end loop;
    not_irq1 <= '1';
    irq2 <= '0';
end process proc_dma;


  
end operate_cl560;
